/**
 * TeilruecknahmeTeilerledigung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.logic.gebuehren;

import java.util.List;

import eu.gronos.kostenrechner.controller.TabulierendZeilen;
import eu.gronos.kostenrechner.controller.TenorDatenContainer;
import eu.gronos.kostenrechner.controller.TenorDatenContainer.HauptsacheEntscheidungsElemente;
import eu.gronos.kostenrechner.controller.TenorDatenContainer.KostenGrundEntscheidungsElemente;
import eu.gronos.kostenrechner.controller.TenorDatenContainer.SonstigeEntscheidungsElemente;
import eu.gronos.kostenrechner.controller.TenorDatenContainer.StreitwertEntscheidungsElemente;
import eu.gronos.kostenrechner.controller.TenorDatenContainer.VollstreckbarkeitsEntscheidungsElemente;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.Allgemein;
import eu.gronos.kostenrechner.controller.VerfahrensDatenContainer.Teilklageruecknahme;
import eu.gronos.kostenrechner.interfaces.Begruendend;
import eu.gronos.kostenrechner.interfaces.TenorVorbereitend;
import eu.gronos.kostenrechner.logic.TenorUtils;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.AuslagenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.HauptsacheVerhaeltnis;
import eu.gronos.kostenrechner.model.tenordaten.KostenTragungsVerhaeltnis;
import eu.gronos.kostenrechner.model.tenordaten.VollstreckbarkeitsListe;
import eu.gronos.kostenrechner.model.tenordaten.VollstreckbarkeitsObjekt;

/**
 * Oberklasse für alle Klasse, die Teilklageerledigung oder Teilklagerücknahme
 * behandeln (bisher Mehrkostenmethode und Quotenmethode). Sie enthält alle
 * gemeinsam benötigten Methoden.
 * 
 * @author Peter Schuster (setrok)
 * @date 16.06.2014
 * 
 */
/**
 * TODO kommentieren: Klasse beschreiben
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 25.10.2018
 *
 */
public abstract class StreitwertReduktion implements TenorVorbereitend, Begruendend {

	static protected final String GRUENDE_EINLEITUNG_91 = "Die Kostenentscheidung beruht auf § 91 ZPO.";
	static protected final String GRUENDE_EINLEITUNG_92 = "Die Kostenentscheidung beruht auf §§ 91, 92 ZPO und ";
	static protected final String GRUENDE_EINLEITUNG_92_269 = "Die Kostenentscheidung beruht auf §§ 91, 92, 269 Abs. 3 ZPO und ";
	// protected static final String GRUENDE_REST =
	// "Zudem wurde von dem verbleibenden Klagegegenstand (mit einer Bedeutung von
	// %,.2f%%) ein Anteil von %,.2f%% nicht zugesprochen.%n";

//	protected final GebuehrenTatbestand[] vorher;
//	protected final GebuehrenTatbestand[] nachher;
	protected final List<GebuehrenTatbestand> vorher;
	protected final List<GebuehrenTatbestand> nachher;
	protected final long[] streitwerte;
	public static final GebuehrenTabelle RVG = new AnwaltsGebuehrenTabelle();
	public static final GebuehrenTabelle GKG = new GerichtsGebuehrenTabelle();
	protected final long verurteilung;
	private final Beteiligter klaeger;
	private final Beteiligter beklagter;
	protected final StringBuilder gruende;
	// protected ArrayList<Object[]> zeilenListe;
	// protected int[] range = new int[2];
	protected TabulierendZeilen zeilen;
	/**
	 * Der quote speichert das Unterliegen des Klägers als double zwischen 0.0 und
	 * 1.0. Vor Initialisierung ist der Wert -1.0.
	 */
	private double quote = -1.0;
	private VollstreckbarkeitsListe vollstreckbarkeitsListe;
	private final boolean b92ii;
	private final TenorDatenContainer container;

	/**
	 * Die Methode berechneUnterliegensQuote dient dazu, die Unterliegensquote nach
	 * Maßgabe der dem Konstruktor übergebenen Werte zu berechnen.
	 * 
	 * @return einen double zwischen 0.0 und 1.0, der in Prozent umgerechnet das
	 *         Unterliegen ausdrückt.
	 */
	protected abstract double berechneUnterliegensQuote();

	/**
	 * Konstruktor:
	 * 
	 * @param teilklageruecknahme Parameterojbkekt vom Typ
	 *                            {@link Teilklageruecknahme}, das enthält die
	 *                            {@link Teilklageruecknahme#parteien} klaeger
	 *                            Informationen zur Klägerseite als Beteiligter
	 *                            (wird zur Tenorierung benötigt) und beklagter
	 *                            Informationen zur Beklagten als Beteiligter (wird
	 *                            zur Tenorierung benötigt), die Arrays
	 *                            gebuehrenVorher und gebuehrenNachher, (Arrays
	 *                            GebuehrenTatbestand[] mit allen schon vor bzw.
	 *                            erst nach der "Streitwertreduktion"
	 *                            (Teilklagerücknahme bzw -erledigung) entstandenen
	 *                            Gebühren, die
	 *                            {@link Teilklageruecknahme#streitwerteUndObsiegen}
	 *                            ein Array long[3] mit streitwerteUndObsiegen[0]
	 *                            als dem alten (höheren) Streitwert,
	 *                            streitwerteUndObsiegen[1] als dem neuen,
	 *                            niedrigeren Streitwert und die
	 *                            streitwerteUndObsiegen[2], die Angabe, in welcher
	 *                            Höhe der Kläger letzlich obsiegt hat.
	 * 
	 * @param allgemein           Paremeterobjekt {@link Allgemein}, das enthält:
	 *                            b92ii Soll § 92 II ZPO angewandt werden
	 *                            (verhältnismäßig geringe Kostentragungen werden
	 *                            unterschlagen)
	 * @throws IllegalArgumentException wird geworfen, wenn eine der beiden
	 *                                  folgenden Plausibilitäten verletzt ist: a)
	 *                                  der frühere Streitwert muss größer als der
	 *                                  spätere (oder gleich) sein, b) der spätere
	 *                                  Streitwert muss größer als die Verurteilung
	 *                                  (oder gleich) sein.
	 */
	public StreitwertReduktion(Allgemein allgemein, Teilklageruecknahme teilklageruecknahme)
			throws IllegalArgumentException {
		super();
		this.streitwerte = new long[] { teilklageruecknahme.streitwerteUndObsiegen.get(0),
				teilklageruecknahme.streitwerteUndObsiegen.get(1) };
		this.verurteilung = teilklageruecknahme.streitwerteUndObsiegen.get(2);
		if (streitwerte == null || streitwerte[1] > streitwerte[0])
			throw new IllegalArgumentException(TenorUtils.FEHLER_REIHENFOLGE_STREITWERTE);
		if (verurteilung > streitwerte[1])
			throw new IllegalArgumentException(TenorUtils.FEHLER_REIHENFOLGE_VERURTEILUNG);
		this.klaeger = teilklageruecknahme.parteien.get(0);// klaeger;
		this.beklagter = teilklageruecknahme.parteien.get(1);// beklagter;
		this.vorher = teilklageruecknahme.vorher;
		this.nachher = teilklageruecknahme.nachher;
		this.b92ii = allgemein.b92ii;
		gruende = new StringBuilder();
		container = new TenorDatenContainer();
		// rvg = new AnwaltsGebuehrenTabelle();
		// gkg = new GerichtsGebuehrenTabelle();
	}
	/*
	 * , Beteiligter klaeger, Beteiligter beklagter, GebuehrenTatbestand[] vorher,
	 * GebuehrenTatbestand[] nachher, long[] streitwerte, long verurteilung, boolean
	 * b92ii
	 * 
	 */

	/**
	 * Die Methode dient dazu, die passende Gebühr oder Auslage zu berechnen.
	 * 
	 * @param gt         einen GebuehrenSatzTatbestand oder AuslagenTatbestand,
	 *                   andere Ableger von GebuehrenTatbestand werden ignoriert.
	 * @param streitwert Streitwert (wird bei Auslagen ignoriert)
	 * @return die Höhe der Gebühr (dann errechnet nach 1,0 Gebühr für den
	 *         streitwert, multipliziert mit dem Gebührensatz nach gt) bzw. Auslage
	 */
	public static double errechneGebuehr(GebuehrenTatbestand gt, long streitwert) {
		if (gt != null && gt instanceof AuslagenTatbestand)
			return ((AuslagenTatbestand) gt).getBetrag();
		else if (gt != null && gt instanceof GebuehrenSatzTatbestand)
			return findeGebuehrenTabelle(gt).errechneGebuehr(streitwert, ((GebuehrenSatzTatbestand) gt).getSatz());
		else
			return 0.0;
	}

	/**
	 * Die Methode erzeugeHauptsachetenor dient dazu, einen einfachen
	 * Hauptsachetenor zu erzeugen.
	 * 
	 * @return ein {@link HauptsacheEntscheidungsElemente}-Objekt mit dem
	 *         Hauptsachetenor als String mit abschließendem Zeilenumbruch.
	 */
	public HauptsacheEntscheidungsElemente erzeugeHauptsacheEntscheidung() {
		HauptsacheEntscheidungsElemente hauptsache = new HauptsacheEntscheidungsElemente();
		if (verurteilung > 0)
			hauptsache.text = TenorUtils.satzBeginn(String.format(
					TenorUtils.VERURTEILUNG_ZAHLUNG_AN_GGF_TEILABWEISUNG_FORMAT,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true),
					klaeger.parteiBezeichner(Beteiligter.AKKUSATIV, klaeger.getLfdNr(), true), (double) verurteilung,
					verurteilung < streitwerte[1] ? TenorUtils.KLAGEABWEISUNG_IM_UEBRIGEN_LEERSCHRITT : ""));
		else
			hauptsache.text = TenorUtils.KLAGEABWEISUNG_ZEILE;
		hauptsache.prozessverhaeltnisse.clear();
		hauptsache.prozessverhaeltnisse.add(new HauptsacheVerhaeltnis(klaeger, beklagter, false, verurteilung));
		return hauptsache;
	}

	/**
	 * Die Methode erzeugeKostentenor dient dazu, einen Kostentenor zu erzeugen. Die
	 * Angaben zu den Parteien werden aus den Konstruktor-Parametern
	 * <code>klaeger</code> und <code>beklagter</code> entnommen, die Quote aus der
	 * Methode {@link berechneUnterliegensQuote()} der abgeleiteten Klassen.
	 * 
	 * @return den Tenortext als String im Format "Die Kosten haben %s zu %,.2f%%
	 *         und %s zu %,.2f%% zu tragen.%n"
	 */
	public KostenGrundEntscheidungsElemente erzeugeKostenEntscheidung() {
		KostenGrundEntscheidungsElemente kostenEntscheidung = new KostenGrundEntscheidungsElemente();
		if (getQuote() == 0.0) {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENTRAGUNG_EINSEITIG,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true));
			kostenEntscheidung.prozessverhaeltnisse.add(new KostenTragungsVerhaeltnis(klaeger, beklagter, false, 1.0));
		} else if (getQuote() == 1.0) {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENTRAGUNG_EINSEITIG,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true));
			kostenEntscheidung.prozessverhaeltnisse.add(new KostenTragungsVerhaeltnis(beklagter, klaeger, false, 1.0));
		} else {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENVERTEILUNG_FORMAT,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true), getQuote() * 100,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true),
					(1.0 - getQuote()) * 100);
			kostenEntscheidung.prozessverhaeltnisse
					.add(new KostenTragungsVerhaeltnis(beklagter, klaeger, false, getQuote()));
			kostenEntscheidung.prozessverhaeltnisse
					.add(new KostenTragungsVerhaeltnis(klaeger, beklagter, false, (1.0 - getQuote())));
		}
		return kostenEntscheidung;
	}

	/**
	 * Die Methode erzeugt den Tenor zur vorläufigen Vollstreckbarkeit
	 * 
	 * @return einen String
	 */
	@Override
	public VollstreckbarkeitsEntscheidungsElemente erzeugeVollstreckbarkeitsEntscheidung() {
		VollstreckbarkeitsEntscheidungsElemente vollEntscheidung = new VollstreckbarkeitsEntscheidungsElemente();
		// TODO vollEntscheidung.prozessverhaeltnisse füllen!
		String schuldner = TenorUtils.JEWEILIGER_VOLLSTRECKUNGSSCHULDNER;
		String glaeubiger = TenorUtils.JEWEILIGER_VOLLSTRECKUNGSGLAEUBIGER;
		if (vollstreckbarkeitsListe == null)
			vollstreckbarkeitsListe = berechneVollstreckbarkeit();
		erweitereGruende(vollstreckbarkeitsListe.toString());
		if (vollstreckbarkeitsListe.sindAlle708iVm713()) {
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR;
			;
			return vollEntscheidung;
		} else if (vollstreckbarkeitsListe.sindAlle709()) {
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709;
			return vollEntscheidung;
		} else if (vollstreckbarkeitsListe.sindAlle708iVm711()) {
			/*
			 * Parteien einsetzen: wenn ein VollstreckbarkeitsObjekt zum Kläger bzw. zum
			 * Beklagten keine Einträge hat, darf nur eine Partei vollstrecken.
			 */
			if (!vollstreckbarkeitsListe.get(1).hatEintraege()) {
				schuldner = klaeger.parteiBezeichner(Beteiligter.DATIV, klaeger.getLfdNr(), true);
				glaeubiger = beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true);
			} else if (!vollstreckbarkeitsListe.get(2).hatEintraege()) {
				schuldner = beklagter.parteiBezeichner(Beteiligter.DATIV, beklagter.getLfdNr(), true);
				glaeubiger = klaeger.parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true);
			}
			/* Ansonsten dürfen beide vollstrecken. */
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR + TenorUtils.satzBeginn(
					String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711, schuldner, glaeubiger));
			return vollEntscheidung;
		}
		/*
		 * Gemischter Vollstreckbarkeitstenor: Beide Vollstreckungsverhältnisse
		 * durchgehen
		 */
		StringBuffer tenor = new StringBuffer(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR);
		Beteiligter[] parteien = new Beteiligter[] { klaeger, beklagter };
		for (int zaehler = 0; zaehler < 2; zaehler++) {
			/* Gläubiger ist, wem der Schleifendurchlauf "gehört". */
			glaeubiger = parteien[zaehler].parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true);
			/*
			 * Schuldner ist jeweils der andere, wird mit Modular-Funktion ("Rest" der
			 * Division durch 2) ermittelt
			 */
			schuldner = parteien[(zaehler + 1) % 2].parteiBezeichner(Beteiligter.DATIV, beklagter.getLfdNr(), true);
			if (vollstreckbarkeitsListe.get(zaehler + 1).isB709s1()) {
				tenor.append(String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709_KOMBI, schuldner));
			} else {
				tenor.append(TenorUtils.satzBeginn(
						String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711, schuldner, glaeubiger)));
			}
		}
		vollEntscheidung.text = tenor.toString();
		return vollEntscheidung;
	}

	/**
	 * Die Methode erzeugt eine Streitwertfestsetzung unter Berücksichtigung des
	 * früheren und des späteren Streitwerts. Das Datum der Reduktion muss man noch
	 * von Hand nachtragen.
	 * 
	 * @return einen String im Format "Der Streitwert wird auf %,.2f EUR bis zum
	 *         ..., danach auf %,.2f EUR festgesetzt.%n"
	 */
	public StreitwertEntscheidungsElemente erzeugeStreitwertEntscheidung() {
		StreitwertEntscheidungsElemente elemente = container.streitwertEntscheidung;
		elemente.text = String.format(TenorUtils.STREITWERTFESTSETZUNG_FORMAT, (double) streitwerte[0],
				(double) streitwerte[1]);
		elemente.streitwerte.add(streitwerte[0]);
		elemente.streitwerte.add(streitwerte[1]);
		return elemente;
	}

	/**
	 * Die Methode erzeugt einen leeren sonstigen Tenor.
	 * 
	 * @return ein {@link SonstigeEntscheidungsElemente}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeSonstigeEntscheidung()
	 */
	@Override
	public SonstigeEntscheidungsElemente erzeugeSonstigeEntscheidung() {
		SonstigeEntscheidungsElemente sonstige = new SonstigeEntscheidungsElemente();
		sonstige.text = "";
		return sonstige;
	}

	/**
	 * Die Methode dient dazu, den gesamten {@link TenorDatenContainer} zu erzeugen.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeContainer()
	 */
	@Override
	public TenorDatenContainer erzeugeContainer() {
		container.berechnungsTyp = getBerechnungsTyp();
		container.hauptsacheEntscheidung = erzeugeHauptsacheEntscheidung();
		container.kostenEntscheidung = erzeugeKostenEntscheidung();
		container.vollstreckbarkeitsEntscheidung = erzeugeVollstreckbarkeitsEntscheidung();
		container.streitwertEntscheidung = erzeugeStreitwertEntscheidung();
		container.sonstigeEntscheidung = erzeugeSonstigeEntscheidung();
		container.begruendung = zeilen.toBegruendungsElemente(getGruende());
		return container;
	}

	/**
	 * @return die Entscheidungsgründe als String
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.Begruendend#getGruende()
	 */
	@Override
	public String getGruende() {
		return gruende.toString();
	}

	/**
	 * @return vorher gibt den vorher als {@link List} zurück
	 */
	List<GebuehrenTatbestand> getVorher() {
		return vorher;
	}

	/**
	 * Die Methode gibt nachher als {@link List} zurück
	 * 
	 * @return
	 */
	List<GebuehrenTatbestand> getNachher() {
		return nachher;
	}

	/**
	 * Getter für {@link #quote}. Wenn der Wert noch nicht gesetzt wurde (dann ist
	 * er < 0.0), wird er über {@link #berechneUnterliegensQuote()} berechnet. Je
	 * nach {@link #b92ii} wird {@link #entferneUnwesentlichesUnterliegen(double)}
	 * aufgerufen.
	 * 
	 * @return gibt {@link #quote} als double zurück.
	 */
	double getQuote() {
		if (quote < 0.0) {
			quote = berechneUnterliegensQuote();
			if (isB92ii())
				quote = entferneUnwesentlichesUnterliegen(quote);
		}
		return quote;
	}

	/**
	 * @return gibt {@link #b92ii} als boolean zurück, also ob § 92 II ZPO angewandt
	 *         werden soll (verhältnismäßig geringe Kostentragungen werden dann
	 *         unterschlagen).
	 */
	boolean isB92ii() {
		return b92ii;
	}

	/**
	 * @return streitwerte gibt den streitwerte als long[] zurück.
	 */
	long[] getStreitwerte() {
		return streitwerte;
	}

	/**
	 * Die Methode dient dazu, &sect;&nbsp;92 Abs.&nbsp;2 ZPO anzuwenden, also eine
	 * verhältnismäßig geringe Kostentragung zu unterschlagen
	 * 
	 * @param quote die Ausgangs-{@link #getQuote() Quote}
	 * @return die bereinigte {@link #getQuote() Quote}
	 */
	private double entferneUnwesentlichesUnterliegen(double quote) {
		if (quote > 0.0 && quote < 0.1)
			return 0.0;
		else if (quote > 0.9)
			return 1.0;
		else
			return quote;
	}

	/**
	 * Die Methode findeGebuehrenTabelle findet heraus, ob ein Gebührentatbestand
	 * zum RVG oder GKG gehört und gibt eine Instanz dieser Tabelle zurück.
	 * 
	 * @param gt der GebuehrenTatbestand
	 * @return die zum GebuehrenTatbestand gehörende GebuehrenTabelle
	 */
	static GebuehrenTabelle findeGebuehrenTabelle(GebuehrenTatbestand gt) {
		if (AnwaltsGebuehrenTabelle.class.equals(gt.getGebuehrenKlasse()))
			return RVG;
		else if (GerichtsGebuehrenTabelle.class.equals(gt.getGebuehrenKlasse()))
			return GKG;
		return null;
	}

	/**
	 * Die Methode berechneAnteileAnRechtsstreit dient dazu, die Anteile des
	 * zurückgenommenen und des verbleibenden Teils am ursprünglichen Streitwert und
	 * damit dem Gesamtrechtsstreits zu ermitteln.
	 * 
	 * @return einen array double[2], mit double-Instanzen zwischen 0.0 und 1.0, bei
	 *         dem double[0] dem Anteil des zurückgenommen Teils am früheren
	 *         Gesamtstreitwert entspricht und double[1] dem verbleibenden Teil am
	 *         früheren Gesamtstreitwert.
	 */
	protected double[] berechneAnteileAnRechtsstreit() {
		double[] zwischen = new double[] { 1.0, 0.0 };
		zwischen[0] = (double) (streitwerte[0] - streitwerte[1]) / (double) streitwerte[0];
		zwischen[1] = (double) streitwerte[1] / (double) streitwerte[0];
		return zwischen;
	}

	/**
	 * Die Methode berechneLetztenVerlust dient dazu, die Verlustquote bzgl. des
	 * niedrigeren Streitwerts nach der Reduktion zu berechnen.
	 * 
	 * @return einen Wert zwischen 0.0 und 1.0, der in Prozente umgerechnet werden
	 *         kann und der Quote entspricht.
	 */
	protected double berechneLetzteVerlustQuote() {
		double zwischen = (double) (streitwerte[1] - verurteilung) / streitwerte[1];
		return zwischen;
	}

	/**
	 * Die Methode berechnet, inwieweit das Urteil vorläufig vollstreckbar ist.
	 * 
	 * @return eine {@link VollstreckbarkeitsListe}, deren 0. Element die
	 *         Unanfechtbarkeit speichert, wenn nicht unanfechtbar das 1. Element
	 *         das Verhältnis kl -> bk und das 2. Element das Verhältnis bk -> kl.
	 */
	private VollstreckbarkeitsListe berechneVollstreckbarkeit() {
		VollstreckbarkeitsListe vollstreckbarkeitsListe = new VollstreckbarkeitsListe();
		vollstreckbarkeitsListe.add(pruefeUnanfechtbarkeit());
		/* Nach festgestellter Unanfechtbarkeit war's das schon. */
		if (vollstreckbarkeitsListe.sindAlle708iVm713())
			return vollstreckbarkeitsListe;
		// if (quote < 0.0)
		// quote = berechneUnterliegensQuote();
		/*
		 * Beim Verhältnis Kl ./. Bk hinsichtlich der Hauptsache einsetzen, was der
		 * Kläger letzlich obsiegt. Hinsichtlich der Kosten die Kosten des Klägers (nach
		 * Streitwert vor Reduktion) einsetzen, soweit sie der Beklagte tragen muss.
		 */
		vollstreckbarkeitsListe.add(VollstreckbarkeitsObjekt.pruefeSicherheitsleistung(verurteilung,
				(long) (TenorUtils.errechneGebuehrenSumme(TenorUtils.KOSTEN_KLAEGER, streitwerte[0])
						* (1.0 - getQuote()))));
		/*
		 * Beim Verhältnis Bk ./. Kl hinsichtlich der Hauptsache 0L einsetzen,
		 * hinsichtlich der Kosten die des Bk einsetzen (Streitwert vor Reduktion),
		 * soweit sie der Kl tragen muss.
		 */
		vollstreckbarkeitsListe.add(VollstreckbarkeitsObjekt.pruefeSicherheitsleistung(0L,
				(long) (TenorUtils.errechneGebuehrenSumme(TenorUtils.KOSTEN_BEKLAGTER, streitwerte[0]) * getQuote())));
		return vollstreckbarkeitsListe;
	}

	/**
	 * Die Methode prüft, ob das Urteil unanfechtbar ist (§§ 708, 713 ZPO).
	 * 
	 * @return ein {@link VollstreckbarkeitsObjekt}, das im Fall der
	 *         Unanfechtbarkeit 708 und 713 auf <code>true</code> setzt. Ansonsten
	 *         enthaelt es nichts.
	 */
	private VollstreckbarkeitsObjekt pruefeUnanfechtbarkeit() {
		/*
		 * Erst einmal das VollstreckbarkeitsObjekt "leer" initialisieren; wenn's nicht
		 * unanfechtbar.
		 */
		VollstreckbarkeitsObjekt unanfechtbarkeit = new VollstreckbarkeitsObjekt();
		/*
		 * Wenn schon der Streitwert vor Reduktion nicht über 600 EUR geht, ist das
		 * Urteil unanfechtbar.
		 */
		if (streitwerte[0] <= TenorUtils.BERUFUNGS_GRENZE) {
			unanfechtbarkeit = new VollstreckbarkeitsObjekt(true, false, false, false, true);
		} else if (streitwerte[0] <= TenorUtils.BERUFUNGS_GRENZE * 2) {
			/*
			 * Wie ist der Kläger beschwert? Das, was von seiner Klage nicht durchgedrungen
			 * ist: Differenz zwischen Streitwert vor Reduktion und letztlich obsiegt. Wie
			 * ist der Beklagte beschwert: Das wozu er verurteilt wurde ... Nur wenn beides
			 * <= 600 ist, dann ist das Urteil unanfechtbar.
			 */
			if (streitwerte[0] - verurteilung <= TenorUtils.BERUFUNGS_GRENZE
					&& verurteilung <= TenorUtils.BERUFUNGS_GRENZE) {
				unanfechtbarkeit = new VollstreckbarkeitsObjekt(true, false, false, false, true);
			}
		}
		return unanfechtbarkeit;
	}

	/**
	 * Die Methode setzt die Entscheidungsgründe zurück (sofern nicht leer) und
	 * beginnt sie neu mit dem übergebenen Text.
	 * 
	 * @param text eine {@link CharSequence} mit dem Anfangssatz(-bestandteil) der
	 *             Gründe.
	 */
	protected void starteGruende(CharSequence text) {
		if (gruende.length() > 0)
			gruende.delete(0, gruende.length());
		erweitereGruende(text);
	}

	/**
	 * Die Methode erweitert die bisherigen Gründe
	 * 
	 * @param text eine {@link CharSequence} mit den weiteren Gründen.
	 */
	void erweitereGruende(CharSequence text) {
		gruende.append(text);
	}

}