/**
 * Forderung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.forderungen;

import java.io.Serializable;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlType;

/**
 * Grundklasse für Forderungen.
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date: 30.07.2014
 *
 */
@XmlType(propOrder = { "wert", "erfolg" })
public
abstract class Forderung implements Serializable {
	private static final long serialVersionUID = 5194917413163124727L;
	/**
	 * Der boolean mit dem Namen gegenfoderung speichert, ob es sich um eine
	 * Forderung des Klägers gegen den Beklagten (false) oder eine Forderung des
	 * Beklagten gegen den Kläger handel
	 */
	private boolean gegenforderung;
	/**
	 * Der double mit dem Namen wert speichert die Höhe der Forderung, also
	 * ihren Wert.
	 */
	private double wert;
	/**
	 * Der double mit dem Namen erfolg speichert, inwieweit die Forderung Erfolg
	 * hatte.
	 */
	private double erfolg;

	/**
	 * Konstruktor:
	 * 
	 * @param gegenforderung
	 *            ob es sich um eine Forderung des Klägers gegen den Beklagten
	 *            (false) oder eine Forderung des Beklagten gegen den Kläger
	 *            handelt
	 * @param wert
	 *            die Höhe der Forderung, also ihren Wert.
	 * @param erfolg
	 *            inwieweit die Forderung Erfolg hatte.
	 * @throws IllegalArgumentException
	 *             wenn der Erfolg höher als der Wert ist
	 */
	Forderung(boolean gegenforderung, double wert, double erfolg)
			throws IllegalArgumentException {
		this.gegenforderung = gegenforderung;
		this.wert = wert;
		setErfolg(erfolg);
	}

	/**
	 * Konstruktor ohne Parameter für {@link JAXB}
	 */
	public Forderung() {
		super();
	}

	/**
	 * @return the gegenforderung
	 */
	@XmlAttribute(name = "gegenforderung")
	boolean isGegenforderung() {
		return gegenforderung;
	}

	/**
	 * @param gegenforderung
	 *            the gegenforderung to set
	 */
	void setGegenforderung(boolean gegenforderung) {
		this.gegenforderung = gegenforderung;
	}

	/**
	 * @return the wert
	 */
	@XmlAttribute(name = "wert")
	public double getWert() {
		return wert;
	}

	/**
	 * @param wert
	 *            the wert to set
	 */
	public void setWert(double wert) {
		this.wert = wert;
	}

	/**
	 * @return the erfolg
	 */
	@XmlAttribute(name = "erfolg")
	public double getErfolg() {
		return erfolg;
	}

	/**
	 * @param erfolg
	 *            the erfolg to set
	 * @throws IllegalArgumentException
	 *             Wenn erfolg höher ist als der Wert der Forderung
	 */
	public void setErfolg(double erfolg) throws IllegalArgumentException {
		if (erfolg > wert)
			throw new IllegalArgumentException(
					"Eine Forderung kann nicht erfolgreicher sein als ihr Wert (ne ultra petita)!");
		else
			this.erfolg = erfolg;
	}

	/**
	 * Eine zu überschreibende Funktion für String-Ausgaben
	 * 
	 * @return eine interne Beschreibung der Forderung für ableitende Klassen.
	 */
	public abstract String getBeschreibung();

	/**
	 * toString:
	 * 
	 * @return
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String.format(
				"Forderung [beschreibung=%s (%s), wert=%s, erfolg=%s]",
				getBeschreibung(), gegenforderung ? "Bk ./. Kl" : "Kl ./. Bk",
				wert, erfolg);
	}

	/**
	 * Berechnet, inwieweit sich zwei Beträge decken, indem es den kleineren von
	 * zwei Beträgen zurückliefert.
	 * 
	 * @param verbliebeneKlageforderung
	 *            was von der Klageforderung übrig blieb
	 * @param andere
	 *            der andere Wert, z.B. den Wert oder den Erfolg
	 * @return das kleinere von beiden oder 0, falls einer der Parameter negativ
	 *         ist.
	 */
	protected static double berechneSchnittmenge(
			double verbliebeneKlageforderung, double andere) {
		if (verbliebeneKlageforderung < 0 || andere < 0)
			return 0.0;
		else
			return verbliebeneKlageforderung < andere ? verbliebeneKlageforderung
					: andere;
	}

}
