package eu.gronos.kostenrechner.view.baumbach;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.baumbach.BaumbachGesamtschuldnerschaft;
import eu.gronos.kostenrechner.model.baumbach.BeteiligtenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;

/**
 * Ein Renderer für Beteiligter, für JTable, JList und JComboBox
 * 
 * @author Peter Schuster (setrok)
 * @date: 27.06.2014
 * 
 * @url "http://docs.oracle.com/javase/tutorial/uiswing/components/table.html"
 */
public class BeteiligtenRenderer extends JLabel implements TableCellRenderer,
		ListCellRenderer<Beteiligter> {
	private static final long serialVersionUID = -7430326486747341015L;
	private final boolean laufendeNummerImmerAusblenden;

	/**
	 * Konstruktor:
	 * 
	 * @param laufendeNummerImmerAusblenden
	 *            wenn "true", so wird der Renderer gezwungen, die laufende
	 *            Nummer beim Generieren des Parteibezeichners auszublenden.
	 *            Ansonsten überprüft der Renderer, ob der darzustellende
	 *            Beteiligte einziger seiner Art ist
	 */
	public BeteiligtenRenderer(boolean laufendeNummerImmerAusblenden) {
		super();
		this.laufendeNummerImmerAusblenden = laufendeNummerImmerAusblenden;
	}

	/**
	 * Standardkonstruktor
	 * 
	 */
	public BeteiligtenRenderer() {
		super();
		this.laufendeNummerImmerAusblenden = false;
	}

	/**
	 * Gibt ein JLabel zum Darstellen der Tabelle zurück. Dabei wird der Text
	 * passend zum übergebenen Beteiligten gesetzt und der Tooltip
	 * gegebenenfalls etwas ausführlicher.
	 * 
	 * @param table
	 *            the <code>JTable</code> that is asking the renderer to draw;
	 *            can be <code>null</code>
	 * @param value
	 *            the value of the cell to be rendered. It is up to the specific
	 *            renderer to interpret and draw the value. For example, if
	 *            <code>value</code> is the string "true", it could be rendered
	 *            as a string or it could be rendered as a check box that is
	 *            checked. <code>null</code> is a valid value
	 * @param isSelected
	 *            true if the cell is to be rendered with the selection
	 *            highlighted; otherwise false
	 * @param hasFocus
	 *            if true, render cell appropriately. For example, put a special
	 *            border on the cell, if the cell can be edited, render in the
	 *            color used to indicate editing
	 * @param row
	 *            the row index of the cell being drawn. When drawing the
	 *            header, the value of <code>row</code> is -1
	 * @param column
	 *            the column index of the cell being drawn
	 * @return the component used for drawing the cell. This method is used to
	 *         configure the renderer appropriately before drawing.
	 * 
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		if (value instanceof BaumbachGesamtschuldnerschaft) {
			/*
			 * Bei Gesamtschuldnerschaft ist es am einfachsten, den richtigen
			 * Parteibezeichner zu finden, weil die
			 * BaumbachGesamtschuldnerschaft ihn selbst vollständig weiß
			 */
			BaumbachGesamtschuldnerschaft bgs = (BaumbachGesamtschuldnerschaft) value;
			setText(bgs.kurzBezeichner(Beteiligter.NOMINATIV));
			setToolTipText(bgs
					.parteiBezeichner(Beteiligter.NOMINATIV, -1, true));
		} else if (value instanceof BaumbachBeteiligter
				&& table.getModel() instanceof BeteiligtenTableModel) {
			/*
			 * Bei anderen BaumbachBeteiligten brauche ich einen Verweis auf die
			 * BaumbachBeteiligtenListe für die laufende Nummer. Deshalb geht
			 * das nur, wenn das TableModel ein BeteiligtenTableModel ist. Dann
			 * kann ich die Arbeit auf die BaumbachBeteiligtenListe abwälzen.
			 */
			BaumbachBeteiligtenListe bbtl = ((BeteiligtenTableModel) table
					.getModel()).getAllValues();
			setText(bbtl.parteiBezeichner(Beteiligter.NOMINATIV, row));
			setToolTipText(bbtl.parteiBezeichner(Beteiligter.NOMINATIV, row));
		} else if (value instanceof Beteiligter) {
			/*
			 * Wenn's ein "ganz einfacher" Beteiligter ist: direkt anzeigen
			 * lassen. Dieser Teil ist übernommen aus SimpleBeteiligtenRenderer
			 */
			Beteiligter b = (Beteiligter) value;
			boolean einzigerSeinerArt = laufendeNummerImmerAusblenden ? laufendeNummerImmerAusblenden
					: table.getRowCount() < 2;
			setText(b.parteiBezeichner(Beteiligter.NOMINATIV, row + 1,
					einzigerSeinerArt));
			setToolTipText(b.parteiBezeichner(Beteiligter.NOMINATIV, row + 1,
					einzigerSeinerArt));
		}
		if (isSelected) {
			/*
			 * Wenn die Zeile ausgewählt ist, müssen die Farben entsprechend dem
			 * selectionBackground und selectionForeground gesetzt werden.
			 */
			setBackground(table.getSelectionBackground());
			setForeground(table.getSelectionForeground());
			setOpaque(true);
		} else {/* sonst entsprechend dem background und foreground der table */
			setBackground(table.getBackground());
			setForeground(table.getForeground());
		}
		return this;
	}

	/**
	 * Die Methode stellt einen Beteiligten in JList und JComboBox dar
	 * 
	 * @param list
	 *            list The JList we're painting.
	 * @param value
	 *            value The value returned by
	 *            list.getModel().getElementAt(index).
	 * @param index
	 *            index The cells index.
	 * @param isSelected
	 *            isSelected True if the specified cell was selected.
	 * @param cellHasFocus
	 *            cellHasFocus True if the specified cell has the focus.
	 * @return A component whose paint() method will render the specified value.
	 * 
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
	 *      java.lang.Object, int, boolean, boolean)
	 */
	@Override
	public Component getListCellRendererComponent(
			JList<? extends Beteiligter> list, Beteiligter value, int index,
			boolean isSelected, boolean cellHasFocus) {
		boolean einzigerSeinerArt = laufendeNummerImmerAusblenden ? laufendeNummerImmerAusblenden
				: list.getModel().getSize() < 2;
		/*
		 * Da Beteiligter.parteiBezeichner die lfdNr zuweist, muss ich
		 * aufpassen, dass ich sie nicht verändere, wenn sie schon gesetzt
		 * wurde. Wenn sie noch nicht gesetzt wurde, ist sie < 0.
		 */
		if (value.getLfdNr() < 0)
			value.setLfdNr(index + 1);
		setText(value.parteiBezeichner(Beteiligter.NOMINATIV, value.getLfdNr(),
				einzigerSeinerArt));
		if (value instanceof BaumbachGesamtschuldnerschaft) {
			BaumbachGesamtschuldnerschaft bgs = (BaumbachGesamtschuldnerschaft) value;
			setToolTipText(bgs.parteiBezeichner(Beteiligter.NOMINATIV,
					value.getLfdNr(), true));
		} else if (value instanceof BaumbachBeteiligter
				|| value instanceof Beteiligter) {
			setToolTipText(value.parteiBezeichner(Beteiligter.NOMINATIV,
					value.getLfdNr(), einzigerSeinerArt));
		}
		if (isSelected) {
			/*
			 * Wenn die Zeile ausgewählt ist, müssen die Farben entsprechend
			 * gesetzt werden.
			 */
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
			setOpaque(true);
		} else {/* sonst nicht */
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		return this;
	}
}